/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version./
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <time.h>
#include "struct.h"
#include "level.h"
#include "draw.h"
#include "manage.h"

#ifdef WIN 
#include "winSpecFce.h"
#else
#include "linSpecFce.h"
#endif


static GTimeVal znacka;
static gint VERSION_FILE[]={VERSION_FILE1,VERSION_FILE2,VERSION_FILE3};
static gint COUNT_LEVEL[]={COUNT_LEVEL1,COUNT_LEVEL2,COUNT_LEVEL3};

static GtkWidget *window;
PangoFontDescription *fontE,*fontEB,*fontB;
static void menuitem_save( gpointer user_data );
static const gchar* TOOLTIPS[]={N_("Move left (left)"),N_("Move up (up)"),
				 N_("Move down (down)"),N_("Move right (right)"),
				 N_("Shooting (space)"),N_("Activate mine (ctrl)"),
				 N_("back (BackSpace)"),N_("Next Round (ins,0)")};
static const gchar* GAME_TYPE[]={N_("Standard"),N_("Advanced"),N_("Hard")};

static GString* getBufferFromFile(GString* file,GError **error)
{
  gchar *buffer;
  gsize length;
  GString* nove;

  if (!g_file_get_contents (file->str, &buffer, &length, error)){
    return NULL;
  }
  nove=g_string_new(buffer);
  g_free(buffer);
  return nove;
} 

static gboolean set_label_font(GtkLabel *label)
{
    PangoLayout *pl;
    
    pl=gtk_label_get_layout(label);
    pango_layout_set_font_description(pl,fontE);
 
    return FALSE;
}

static gboolean set_label_font_s(GtkLabel *label)
{
    PangoLayout *pl;
    
    pl=gtk_label_get_layout(label);
    pango_layout_set_font_description(pl,fontB);
 
    return FALSE;
}

static gboolean set_label_fontB(GtkLabel *label)
{
    PangoLayout *pl;
    
    pl=gtk_label_get_layout(label);
    pango_layout_set_font_description(pl,fontEB);
 
    return FALSE;
}

static void focusFirstButton(GtkWidget* actionArea)
{
    GList* childern=GTK_BOX(actionArea)->children;
    GtkWidget* button=/*GTK_BOX_CHILD*/((GtkBoxChild*)(childern->data))->widget;
    
    gtk_widget_grab_focus(button);
}    

static void editFileInDialog(const gchar* file,const gchar* nadpis)
{
    GString* buffer;
    GString* dir=g_string_new("");
    GError *error=NULL;
    GtkWidget* dialog;
    GtkWidget* vbox;
    GtkWidget* scroll;
    GtkWidget* edit;
    GtkTextBuffer *tbuffer;

#ifdef HAVE_CONFIG_H
#ifdef RELATIVE_PATH
    g_string_append(dir,"..");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"share");
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#else
    g_string_append(dir,PACKAGE_DATA_DIR);
    g_string_append_c(dir,G_DIR_SEPARATOR);
    g_string_append(dir,"babala");
    g_string_append_c(dir,G_DIR_SEPARATOR);
#endif     
#endif
    g_string_append(dir,file);
   
    buffer=getBufferFromFile(dir,&error); 
    g_string_free(dir,TRUE); 
    if(!buffer){
        return;  
    }
    
    dialog = gtk_dialog_new_with_buttons (nadpis,GTK_WINDOW(window),
                                         GTK_DIALOG_MODAL |GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_OK, GTK_RESPONSE_NONE,
                                         NULL);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
    gtk_widget_set_size_request (GTK_WIDGET (dialog),400, 300);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
    
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
    
    scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scroll);
    gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 2);

    edit = gtk_text_view_new ();
    gtk_widget_show (edit);
    gtk_container_add (GTK_CONTAINER (scroll), edit);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (edit), FALSE);
   
    tbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
    gtk_text_buffer_set_text(tbuffer,buffer->str,buffer->len);
    
    g_string_free(buffer,TRUE);
    
    focusFirstButton(GTK_DIALOG (dialog)->action_area);
    
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);  
}

static void editState(Level* level)
{
    GtkWidget* dialog;
    GtkWidget* vbox,*separator;
    GtkWidget *table1,*table2,*label;
    gchar popis[20];
    gint countBall[5];
    gint countSh[2];
    gint countMin[3]; 
    
    getPoctyAll(level,countBall, countSh,countMin);
    dialog = gtk_dialog_new_with_buttons (_("State of level"),GTK_WINDOW(window),
                                         GTK_DIALOG_MODAL |GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_OK, GTK_RESPONSE_NONE,
                                         NULL);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
    
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
    
    table1 = gtk_table_new (7,3,TRUE);
    gtk_widget_show (table1);
    gtk_box_pack_start (GTK_BOX (vbox), table1, FALSE, FALSE, 10);

    label=setCellSize(table1,0,1,0,1,_("Balabaks"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table1,0,1,1,2,_("Blue"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table1,0,1,2,3,_("Green"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table1,0,1,3,4,_("Red"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
	label=setCellSizeOd(table1,0,1,4,5,_("Black"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table1,0,1,5,6,_("Orange"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSize(table1,0,1,6,7,_("Total"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);

    label=setCellSize(table1,1,2,0,1,_("Count"),FALSE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    g_snprintf(popis,19,"%d",countBall[0]);
    setCell(table1,1,2,1,2,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[1]);
    setCell(table1,1,2,2,3,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[2]);
    setCell(table1,1,2,3,4,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[3]);
    setCell(table1,1,2,4,5,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[4]);
    setCell(table1,1,2,5,6,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[0]+countBall[1]+countBall[2]+countBall[3]+countBall[4]);
    setCell(table1,1,2,6,7,popis,FALSE);

    label=setCellSize(table1,2,3,0,1,_("Lives"),FALSE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    g_snprintf(popis,19,"%d",countBall[0]);
    setCell(table1,2,3,1,2,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[1]);
    setCell(table1,2,3,2,3,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[2]*2);
    setCell(table1,2,3,3,4,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[3]);
    setCell(table1,2,3,4,5,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[4]*3);
    setCell(table1,2,3,5,6,popis,FALSE);
    g_snprintf(popis,19,"%d",countBall[0]+countBall[1]+countBall[2]*2+countBall[3]+3*countBall[4]);
    setCell(table1,2,3,6,7,popis,FALSE);

    separator = gtk_hseparator_new(); 
    gtk_box_pack_start (GTK_BOX (vbox), separator, FALSE, FALSE, 2);
    gtk_widget_show (separator);   
  
    table2 = gtk_table_new (5,4,TRUE);
    gtk_widget_show (table2);
    gtk_box_pack_start (GTK_BOX (vbox), table2, FALSE, FALSE, 10);

    label=setCellSize(table2,0,1,0,1,_("Munitions"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table2,0,1,1,2,_("Owned"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table2,0,1,2,3,_("Cached"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSizeOd(table2,0,1,3,4,_("Live Mines"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    label=setCellSize(table2,0,1,4,5,_("Total"),TRUE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);

    label=setCellSize(table2,1,2,0,1,_("Projectiles"),FALSE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    g_snprintf(popis,19,"%d",countSh[0]);
    setCell(table2,1,2,1,2,popis,FALSE);
    g_snprintf(popis,19,"%d",countSh[1]);
    setCell(table2,1,2,2,3,popis,FALSE);
    setCell(table2,1,2,3,4,"---",FALSE);
    g_snprintf(popis,19,"%d",countSh[0]+countSh[1]);
    setCell(table2,1,2,4,5,popis,FALSE);

    label=setCellSize(table2,2,3,0,1,_("Mines"),FALSE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    g_snprintf(popis,19,"%d",countMin[0]);
    setCell(table2,2,3,1,2,popis,FALSE);
    g_snprintf(popis,19,"%d",countMin[1]);
    setCell(table2,2,3,2,3,popis,FALSE);
    g_snprintf(popis,19,"%d",countMin[2]);
    setCell(table2,2,3,3,4,popis,FALSE);
    g_snprintf(popis,19,"%d",countMin[0]+countMin[1]+countMin[2]);
    setCell(table2,2,3,4,5,popis,FALSE);

    label=setCellSize(table2,3,4,0,1,_("Total"),FALSE,100,20);
    g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font_s),label);
    g_snprintf(popis,19,"%d",countSh[0]+countMin[0]);
    setCell(table2,3,4,1,2,popis,FALSE);
    g_snprintf(popis,19,"%d",countSh[1]+countMin[1]);
    setCell(table2,3,4,2,3,popis,FALSE);
    g_snprintf(popis,19,"%d",countMin[2]);
    setCell(table2,3,4,3,4,popis,FALSE);
    g_snprintf(popis,19,"%d",countMin[0]+countMin[1]+countMin[2]+
	       countSh[0]+countSh[1]);
    setCell(table2,3,4,4,5,popis,FALSE);
    focusFirstButton(GTK_DIALOG (dialog)->action_area);
    
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);  
}

static void editLegend()
{
    GtkWidget* dialog;
    GtkWidget* vbox;
    GtkWidget *table;
	GtkWidget *scroll,*viewport;
	guint32 ktere=0;
	gint i;    
	for(i=0;i<25;i++) ktere |=(guint32)(1<<i);

    dialog = gtk_dialog_new_with_buttons (_("Legend"),GTK_WINDOW(window),
                                         GTK_DIALOG_MODAL |GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_OK, GTK_RESPONSE_NONE,
                                         NULL);
	gtk_widget_set_size_request (GTK_WIDGET (dialog),700,500);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
    
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);

    scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scroll);
    gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
         GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
    
    viewport =gtk_viewport_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (scroll), viewport);
    gtk_widget_show (viewport);
    
    table = createTableTips(ktere);
    gtk_widget_show (table);
	gtk_container_add (GTK_CONTAINER (viewport), table);
	
	focusFirstButton(GTK_DIALOG (dialog)->action_area);
    
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);  
}

static void editScore(Level* level)
{
    GList* score=nactiScore(level->info->file);
    GtkWidget* dialog;
    GtkWidget* vbox;  
    
    dialog = gtk_dialog_new_with_buttons (_("Score"),GTK_WINDOW(window),
                                         GTK_DIALOG_MODAL |GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_OK,
                                         GTK_RESPONSE_NONE,
                                         NULL);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
    gtk_widget_set_size_request (GTK_WIDGET (dialog),400, 400);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
    
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
 
    zobrazScore(score,GTK_BOX(vbox));
    uvolniScore(score);
    
    focusFirstButton(GTK_DIALOG (dialog)->action_area);
    
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);  
}  

static void editScoreL(Level* level)
{
    GtkWidget* dialog;
    GtkWidget* vbox;  
    
    dialog = gtk_dialog_new_with_buttons (_("Levels Score"),GTK_WINDOW(window),
                                         GTK_DIALOG_MODAL |GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_OK,
                                         GTK_RESPONSE_NONE,
                                         NULL);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
    gtk_widget_set_size_request (GTK_WIDGET (dialog),400, 400);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
    
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
 
    zobrazScoreLAll(level->info->file,GTK_BOX(vbox));

    focusFirstButton(GTK_DIALOG (dialog)->action_area);
    
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);  
}  

static GString* getHomeDir()
{
    GtkWidget* dialog;
    
    const gchar* dir=g_get_home_dir();
    GString* path;
 
   if(!dir){
        path=g_string_new(".babala");
   }else{
        gchar* vysl;
        vysl=g_build_filename(dir,".babala",NULL);
        path=g_string_new(vysl);
        g_free(vysl);
    }
    
    if(!g_file_test(path->str,G_FILE_TEST_EXISTS)){
        mkdir(path->str,S_IRUSR | S_IWUSR | S_IXUSR);
    }
    if(!g_file_test(path->str,G_FILE_TEST_EXISTS)){
        gchar pom[200];
        
        g_snprintf(pom,199,_("Foder %s isn't created."),path->str);
        g_string_free(path,TRUE);
        dialog =gtk_message_dialog_new (GTK_WINDOW(window),
    		     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
    			 GTK_MESSAGE_ERROR,GTK_BUTTONS_CLOSE,"%s",pom);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
        return NULL; 
    }
    return path;    
}

static void zapisStav(Level* level)
{
    FILE* file;
    GString* homeFile=getHomeDir();
    CodeScore code;
    Score score;
    gint version=VERSION_H;
    GTimeVal result;
     
    level->info->max=(gboolean)(
        gdk_window_get_state(window->window) & GDK_WINDOW_STATE_MAXIMIZED );
    gtk_window_get_size(GTK_WINDOW(window),&level->info->windx,&level->info->windy);
    if(!homeFile)return;
    g_string_append_c(homeFile,G_DIR_SEPARATOR); 
    g_string_append(homeFile,"state");
    file=fopen(homeFile->str,"wb");
    if(!file) return;
    if(fwrite((void*)&version,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->file,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->maxPor,sizeof(guint32),1,file)==0)return;
    if(fwrite((void*)&level->info->name->len,sizeof(gsize),1,file)==0)return;
    if(fwrite((void*)level->info->name->str,sizeof(gchar),
        level->info->name->len,file)==0)return;
    if(fwrite((void*)&level->info->animate,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->beep,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->persRange,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->windx,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->windy,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->max,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->tip,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->less_scale,sizeof(gboolean),1,file)==0)return;
    fclose(file);

#ifdef OPEN_SOURCE
    g_string_append(homeFile,"-os");
#endif
    
    switch(level->info->file){
	case 2:
		g_string_append(homeFile,"3");
        break;
	case 1:
		g_string_append(homeFile,"2");
        break;
    default:
        g_string_append(homeFile,"1");
        break;
    } 
  
    file=fopen(homeFile->str,"wb");
    g_string_free(homeFile,TRUE);
    if(!file) return;
    version=VERSION_FILE[level->info->file];
    if(fwrite((void*)&version,sizeof(gint),1,file)==0)return;
    
    g_get_current_time(&result);
    score.time=(GTime)result.tv_sec;
    score.code=level->info->code;
    score.por=level->info->por;
    score.score=level->info->iscore;
    score.level=level->level;
    kodujScore(&code,score);
    
    if(fwrite((void*)&code,sizeof(guint8),32,file)==0)return;
    
    score.score=level->best;
    score.por+=12;
    score.level*=2;
    kodujScore(&code,score);
    
    if(fwrite((void*)&code,sizeof(guint8),32,file)==0)return;
	
    fclose(file);
}

static void zapisName(Level* level)
{
    FILE* file;
    GString* homeFile=getHomeDir();
    gint version=VERSION_H;
    
    level->info->max=(gboolean)(gdk_window_get_state(window->window) & 
        GDK_WINDOW_STATE_MAXIMIZED );
	gtk_window_get_size (GTK_WINDOW(window),&level->info->windx,&level->info->windy);
    if(!homeFile)return;
    g_string_append_c(homeFile,G_DIR_SEPARATOR); 
    g_string_append(homeFile,"state");
    
    file=fopen(homeFile->str,"wb");
    g_string_free(homeFile,TRUE);
    if(!file) return;
    if(fwrite((void*)&version,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->file,sizeof(guint32),1,file)==0)return;
    if(fwrite((void*)&level->info->maxPor,sizeof(guint32),1,file)==0)return;
    if(fwrite((void*)&level->info->name->len,sizeof(gsize),1,file)==0)return;
    if(fwrite((void*)level->info->name->str,sizeof(gchar),
        level->info->name->len,file)==0)return;
    if(fwrite((void*)&level->info->animate,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->beep,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->persRange,sizeof(gboolean),1,file)==0)return;

    if(fwrite((void*)&level->info->windx,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->windy,sizeof(gint),1,file)==0)return;
    if(fwrite((void*)&level->info->max,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->tip,sizeof(gboolean),1,file)==0)return;
    if(fwrite((void*)&level->info->less_scale,sizeof(gboolean),1,file)==0)return;
    fclose(file);
}

static void redrawMap(GtkWidget* widget,Level* level)
{
    GdkRectangle rectangle;
    gint cx,cy;
    gint rozmer=level->info->less_scale?20:32;
 
    gtk_widget_set_size_request(widget,level->map.columns*rozmer+1,
                                level->map.raws*rozmer+1);
    rectangle.x=0;
    rectangle.y=0;
    gdk_drawable_get_size(widget->window,&cx,&cy);
    rectangle.width=cx;
    rectangle.height=cy;
    gdk_window_invalidate_rect(widget->window,&rectangle,TRUE);
}

static void editNameDialog(Level* level,gboolean widthCheck)
{
    GtkWidget* dialog;
    GtkWidget* vbox;
    GtkWidget* label;
    GtkWidget* entry;
    GtkWidget* animate=NULL;
    GtkWidget* beep=NULL;
	GtkWidget* tip=NULL;
	GtkWidget* persRange=NULL;
    GString* napis=g_string_new(level->info->name->str);
    gboolean isRange=level->info->persRange;
    const gchar* un=g_get_user_name();

    if(napis->len==0){
        g_string_free(napis,TRUE);
        napis=g_string_new(un);
    }    
    dialog = gtk_dialog_new_with_buttons (_("Options"),GTK_WINDOW(window),
                                          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,
                                          GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,NULL);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
    gtk_widget_set_size_request (GTK_WIDGET (dialog),300, widthCheck?200:100);
     
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
    
    label = gtk_label_new (widthCheck?_("Name for score:"):_("Set yours name for score:"));
    gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 2);
    gtk_widget_show (label);
    
    entry=gtk_entry_new();
    gtk_box_pack_start(GTK_BOX (vbox), entry, TRUE, TRUE, 2);
    gtk_entry_set_text(GTK_ENTRY(entry),napis->str);
    gtk_widget_show (entry);
    
    if(widthCheck){
        animate=gtk_check_button_new_with_label(_("Animations"));
        gtk_box_pack_start(GTK_BOX (vbox), animate, TRUE, TRUE, 2);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(animate),level->info->animate);
        gtk_widget_show (animate);
        
        beep=gtk_check_button_new_with_label(_("Beep"));
        gtk_box_pack_start(GTK_BOX (vbox),beep , TRUE, TRUE, 2);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(beep),level->info->beep);
        gtk_widget_show (beep);

		persRange=gtk_check_button_new_with_label(_("Show Range permanently"));
        gtk_box_pack_start(GTK_BOX (vbox),persRange , TRUE, TRUE, 2);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(persRange),level->info->persRange);
        gtk_widget_show (persRange);

		tip=gtk_check_button_new_with_label(_("Show Tips"));
        gtk_box_pack_start(GTK_BOX (vbox),tip , TRUE, TRUE, 2);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tip),level->info->tip);
        gtk_widget_show (tip);
    }        
    
    if(gtk_dialog_run (GTK_DIALOG (dialog))==GTK_RESPONSE_ACCEPT){
        const gchar* nove=gtk_entry_get_text(GTK_ENTRY(entry) );    
        g_string_free(level->info->name,TRUE);
        level->info->name=g_string_new(nove);
        if(widthCheck){
            level->info->animate=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(animate));
            level->info->beep=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(beep));
			level->info->persRange=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(persRange));
			level->info->tip=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(tip));
        }    
        zapisName(level);
    }    
    gtk_widget_destroy (dialog);  
    g_string_free(napis,TRUE);
    
    if(!isRange && level->info->persRange){
         vyberAll(level);
         redrawMap(GTK_WIDGET(level->info->area),level);
    }    
}

static void showTip(Level* level)
{
    GtkWidget* dialog;
    GtkWidget* vbox;
    GtkWidget *table;
	GtkWidget* tip;
	
    if(!level->info->tip || level->tip==0) return;

    dialog = gtk_dialog_new_with_buttons (_("Tips"),GTK_WINDOW(window),
                                          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_STOCK_OK,GTK_RESPONSE_NONE,NULL);
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));
     
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
    
    vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (vbox);
 
    table = createTableTips(level->tip);
    gtk_widget_show (table);

    gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);

	tip=gtk_check_button_new_with_label(_("Show Tips"));
    gtk_box_pack_start(GTK_BOX (vbox),tip , FALSE,FALSE, 2);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tip),TRUE);
    gtk_widget_show (tip);
    
    focusFirstButton(GTK_DIALOG (dialog)->action_area);
    
    gtk_dialog_run (GTK_DIALOG (dialog));

	level->info->tip=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(tip));
    if(!level->info->tip)zapisName(level);
    
    gtk_widget_destroy (dialog);  
}
    
static gboolean nastavZacatek(Level* level)
{
    FILE* file;
    size_t kontr;
    GString* homeFile=getHomeDir();
    gsize vel=0;
    gint version;
    CodeScore code;
    Score score;
    
    if(!homeFile)return FALSE;
    g_string_append_c(homeFile,G_DIR_SEPARATOR); 
    g_string_append(homeFile,"state");
    file=fopen(homeFile->str,"rb");
    if(!file) return FALSE;
    kontr=fread((void*)&version,sizeof(gint),1,file);
    if(!kontr || version<200){
        fclose(file);
        return FALSE;
    }    
    kontr=fread((void*)&level->info->file,sizeof(gint),1,file);
    if(!kontr || level->info->file>MAX_FILE || level->info->file<0){
        fclose(file);
        return FALSE;
    }
    kontr=fread((void*)&level->info->maxPor,sizeof(guint32),1,file);
    if(!kontr){
        fclose(file);
        return FALSE;
    } 
    kontr=fread((void*)&vel,sizeof(gsize),1,file);
	level->info->name=g_string_new_len("",vel+1);
    g_string_set_size(level->info->name,vel);
    kontr=fread((void*)level->info->name->str,sizeof(gchar),vel,file);
    kontr=fread((void*)&level->info->animate,sizeof(gboolean),1,file);
    if(!kontr)level->info->animate=TRUE;
    kontr=fread((void*)&level->info->beep,sizeof(gboolean),1,file);
    if(!kontr)level->info->beep=FALSE;
	kontr=fread((void*)&level->info->persRange,sizeof(gboolean),1,file);
    if(!kontr)level->info->persRange=FALSE;
	kontr=fread((void*)&level->info->windx,sizeof(gint),1,file);
    if(!kontr)level->info->windx=800;
	kontr=fread((void*)&level->info->windy,sizeof(gint),1,file);
    if(!kontr)level->info->windy=600;
    kontr=fread((void*)&level->info->max,sizeof(gboolean),1,file);
    if(!kontr){
        level->info->max=FALSE;
    }else{
        if(level->info->max){
            level->info->windx=800;
            level->info->windy=600;
        }    
    } 
	kontr=fread((void*)&level->info->tip,sizeof(gboolean),1,file);
    if(!kontr)level->info->tip=TRUE;
	kontr=fread((void*)&level->info->less_scale,sizeof(gboolean),1,file);
	if(!kontr)level->info->less_scale=FALSE;

    fclose(file);
    
#ifdef OPEN_SOURCE
	g_string_append(homeFile,"-os");
#endif

    switch(level->info->file){
	case 2:
		g_string_append(homeFile,"3");
        break;
	case 1:
		g_string_append(homeFile,"2");
        break;
    default:
        g_string_append(homeFile,"1");
        break;
    }
    file=fopen(homeFile->str,"rb");
    g_string_free(homeFile,TRUE);
    if(!file){
    	GRand* rand;
	    rand=g_rand_new ();
	    level->info->por=g_rand_int(rand)/37;
	    level->info->maxPor=level->info->por+1;
	    level->info->code=g_rand_int(rand);
    	level->level=1;
    }else{
        kontr=fread((void*)&version,sizeof(gint),1,file);
     
    	kontr=fread((void*)&code,sizeof(guint8),32,file);
    	if(kontr!=32)return FALSE;
    	if(!dekodujScore(&score,code))return FALSE;
    
    	level->info->code=score.code;
    	level->info->por=score.por;
    	level->info->iscore=score.score;
    	level->level=score.level;
    
    	kontr=fread((void*)&code,sizeof(guint8),32,file);
    	if(kontr!=32)return FALSE;
    	if(!dekodujScore(&score,code) || level->info->por+12 != score.por ||
        	2*level->level!=score.level || version!=VERSION_FILE[level->info->file])    {
        	    fclose(file);
        	    return FALSE;
    	}    
    	fclose(file);
	}

    setLevelSet(level->info->file);
    
    level->best=score.score;
    level->lastHistory=0;
    level->oznacB=NULL;
    level->oznacP=NULL;
    level->isPlayerO=FALSE;
    getLevel(level,level->level);
 //   getLevel(level,83);
    return TRUE;
}    

static gboolean nastavZacatekNew(Level* level)
{
    GRand* rand;
    
    level->info->file=0;
    level->info->iscore=(guint32)0;
    level->info->name=g_string_new("");
    level->lastHistory=0;
    level->info->animate=TRUE;
    level->info->beep=FALSE;
	level->info->tip=TRUE;
    level->oznacB=NULL;
    level->oznacP=NULL;
    level->isPlayerO=FALSE;
	level->info->persRange=FALSE;
	level->info->windx=800;
	level->info->windy=600;
	level->info->max=FALSE;
	level->info->less_scale=FALSE;
	level->best=0;
        
    rand=g_rand_new ();
    level->info->por=g_rand_int(rand)/37;
    level->info->maxPor=level->info->por+1;
    level->info->code=g_rand_int(rand);
    g_rand_free (rand);

	setLevelSet(level->info->file);
    getLevel(level,1);
    return TRUE;
}

static void on_repeat_clicked(GtkButton *button, gpointer user_data)
{
    GtkDialog *dialog=GTK_DIALOG(user_data);
    gtk_dialog_response (dialog,(gint)0);
}

static void on_next_clicked(GtkButton *button, gpointer user_data)
{
    GtkDialog *dialog=GTK_DIALOG(user_data);
    gtk_dialog_response (dialog,(gint)1);
}

static void on_quit_clicked(GtkButton *button, gpointer user_data)
{
    GtkDialog *dialog=GTK_DIALOG(user_data);
    gtk_dialog_response (dialog,(gint)2);
}

static gboolean on_key_event_d(GtkWidget *widget,GdkEventKey *event,
                                      gpointer user_data)
{
//    Level* level=(Level*)user_data;

    switch(event->keyval){
    case GDK_space:
        if(!porovnejCas(&znacka))return TRUE;
        break;
    }    
    return FALSE;
}


static GtkWidget*
create_endLevel(gboolean winPlayer,gboolean isMun, Level* level)
{
    GtkWidget *endLevel;
    GtkWidget *dialog_vbox;
    GtkWidget *label;
    GtkWidget *dialog_action_area1;
    GtkWidget *repeat;
    GtkWidget *next;
    GtkWidget *quit;
    
    endLevel = gtk_dialog_new ();
    gtk_window_set_transient_for(GTK_WINDOW(endLevel),GTK_WINDOW(window));

    gtk_window_set_title (GTK_WINDOW (endLevel), _("End Level"));
    gtk_window_set_modal (GTK_WINDOW (endLevel), TRUE);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (endLevel), TRUE);
    
    dialog_vbox = GTK_DIALOG (endLevel)->vbox;
    gtk_widget_show (dialog_vbox);
    
    if(winPlayer){
        label = gtk_label_new (_("You Win."));
    }else{
        if(isMun)
            label = gtk_label_new (_("You Lose."));   
        else
            label = gtk_label_new (_("You can't Win. (You haven't enough munitions)"));   
    }    
    gtk_box_pack_start (GTK_BOX (dialog_vbox), label, FALSE, FALSE, 0);
    gtk_widget_show (label);
    gtk_misc_set_padding (GTK_MISC (label), 25, 10);
    
    if(winPlayer){
        zobrazScoreL(level->info->file,GTK_BOX (dialog_vbox),level,
                set_label_font,set_label_fontB);
    }else{
        if(level->best>0){
            gchar popis[20];
                        
            label = gtk_label_new (_("But your score of record from earlier\nplay is not forfeited, and remains:"));
            gtk_box_pack_start (GTK_BOX (dialog_vbox), label, FALSE, FALSE, 0);
            gtk_widget_show (label);
            
            g_snprintf(popis,19,"%d",level->best);
            label = gtk_label_new (popis);
            gtk_widget_set_size_request(GTK_WIDGET(label),100,25);
            gtk_box_pack_start (GTK_BOX (dialog_vbox), label, FALSE, FALSE, 5);
            gtk_misc_set_padding (GTK_MISC (label), 5, 5);
            gtk_widget_show (label);
            g_signal_connect(label, "expose-event", G_CALLBACK(set_label_font),label);
        }    
    }        
    
    dialog_action_area1 = GTK_DIALOG (endLevel)->action_area;
    gtk_widget_show (dialog_action_area1);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
    
    repeat = gtk_button_new_with_mnemonic (_("Repeat Level"));
    gtk_dialog_add_action_widget (GTK_DIALOG (endLevel),repeat, 0);
    if(!winPlayer && level->best>0)gtk_widget_grab_focus(repeat);
    gtk_widget_show (repeat);
    GTK_WIDGET_SET_FLAGS (repeat, GTK_CAN_DEFAULT);
    
    g_signal_connect ((gpointer) repeat, "clicked",
                    G_CALLBACK (on_repeat_clicked),
                    (gpointer)endLevel);
    
    if(winPlayer || level->best>0){
        next = gtk_button_new_with_mnemonic (_("Next Level"));
        gtk_widget_show (next);
        gtk_dialog_add_action_widget (GTK_DIALOG (endLevel), next, 0);
        GTK_WIDGET_SET_FLAGS (next, GTK_CAN_DEFAULT);
        if(winPlayer)gtk_widget_grab_focus(next);
        
        g_signal_connect ((gpointer) next, "clicked",
                G_CALLBACK (on_next_clicked),
                (gpointer)endLevel);
    }else{
        quit = gtk_button_new_with_mnemonic (_("Exit Babala"));    
        gtk_widget_show (quit);
        gtk_dialog_add_action_widget (GTK_DIALOG (endLevel), quit, 0);
        GTK_WIDGET_SET_FLAGS (quit, GTK_CAN_DEFAULT);
            
        g_signal_connect ((gpointer) quit, "clicked",
                        G_CALLBACK (on_quit_clicked),
                        (gpointer)endLevel);
    }
    

    g_get_current_time(&znacka);
    g_signal_connect ((gpointer)endLevel, "key_press_event",
                      G_CALLBACK (on_key_event_d),
                      (gpointer)&level);
    return endLevel;
}

static void setStatus(Level* level)
{
    gchar cislo[16];
    
    g_snprintf(cislo,15,"%d",level->player.moves);
    gtk_entry_set_text( level->info->moves1,cislo );

    g_snprintf(cislo,15,"%d",level->player.shots);
    gtk_entry_set_text( level->info->moves2,cislo );
    
    g_snprintf(cislo,15,"%d",level->player.projectiles);
    gtk_entry_set_text( level->info->projectile1,cislo );

    g_snprintf(cislo,15,"%d",level->player.mines);
    gtk_entry_set_text( level->info->projectile2,cislo );
    
    g_snprintf(cislo,15,"%d",level->level);
    gtk_entry_set_text( level->info->score1,cislo );
    
    g_snprintf(cislo,15,"%d",level->info->iscore);
    gtk_entry_set_text( level->info->score2,cislo );
    
    if(level->player.orientation ==3 &&
       (level->player.moves==0 || !kontrola_move(3,level)))
        gtk_widget_set_sensitive(level->info->butt[0],FALSE);
    else
        gtk_widget_set_sensitive(level->info->butt[0],TRUE);
    if(level->player.orientation ==0 &&
       (level->player.moves==0 || !kontrola_move(0,level)))
        gtk_widget_set_sensitive(level->info->butt[1],FALSE);
    else
        gtk_widget_set_sensitive(level->info->butt[1],TRUE);
    if(level->player.orientation ==2 &&
       (level->player.moves==0 || !kontrola_move(2,level)))
        gtk_widget_set_sensitive(level->info->butt[2],FALSE);
    else
        gtk_widget_set_sensitive(level->info->butt[2],TRUE);
    if(level->player.orientation ==1 &&
       (level->player.moves==0 || !kontrola_move(1,level)))
        gtk_widget_set_sensitive(level->info->butt[3],FALSE);
    else
        gtk_widget_set_sensitive(level->info->butt[3],TRUE);

    if(level->player.shots==0 || level->player.projectiles==0)
        gtk_widget_set_sensitive(level->info->butt[4],FALSE);
    else
        gtk_widget_set_sensitive(level->info->butt[4],TRUE);

    if(level->player.shots==0 || !kontrola_mina(level))
        gtk_widget_set_sensitive(level->info->butt[5],FALSE);
    else
        gtk_widget_set_sensitive(level->info->butt[5],TRUE);

    if(is_history(level)) gtk_widget_set_sensitive(level->info->butt[6],TRUE);
    else gtk_widget_set_sensitive(level->info->butt[6],FALSE); 
}

static void set_level_set(gpointer user_data , gint levelSet)
{
    Level* level=(Level*)user_data;

	if(level->info->file == levelSet) return;

	zapisStav(level);
    level->info->file=levelSet;
    zapisName(level);
    level_free(level,FALSE);

    nastavZacatek(level); 
    setStatus(level);
	if(level->info->persRange)	vyberAll(level);
    redrawMap(GTK_WIDGET(level->info->area),level);
}

static void ukonciLevel(gboolean winPlayer,Level* level,gboolean isMun)
{
    GtkWidget* dialog;
    gint nextLevel;
    gint result;
    
    dialog=create_endLevel(winPlayer,isMun,level);
    result =gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  
    if(!winPlayer && result==2 && level->best<1){
      gtk_main_quit();
      return;
    }
  
    if(result==1){
        GList* score=nactiScore(level->info->file);
        
        level->info->iscore+=level->best;
        gtk_widget_set_sensitive(level->info->skip_item,level->info->iscore>49);
        
        nextLevel=level->level+1;
        if(level->info->name->len==0) editNameDialog(level,FALSE);
        pridejZapisDoScore(&score,level->info->name,level->info->iscore,
                level->level,level->info->por,level->info->code);
        if(winPlayer)ulozScoreL(level->info->file,level);
        level->level++;
        level->best=0;
        zapisStav(level);
        zapisScore(level->info->file,score);
        uvolniScore(score);
    }else{
        nextLevel=level->level;
        if(winPlayer)ulozScoreL(level->info->file,level);
        zapisStav(level);
    }  
    
    if(nextLevel==COUNT_LEVEL[level->info->file]+1){
        GtkWidget* dialog2;
        GRand* rand;
        
        level_free(level,FALSE);
        level->best=0;
        level->info->por=level->info->maxPor;
        level->info->maxPor++;
        level->info->iscore=(guint32)0;
        rand=g_rand_new ();
        level->info->code=g_rand_int(rand);
        g_rand_free (rand);
        getLevel(level,1);
        
        setStatus(level);
        zapisStav(level);
        dialog2 =gtk_message_dialog_new (GTK_WINDOW(window),
    		     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
    			 GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,
    			 _("YOU WIN !!!!"));
        gtk_dialog_run (GTK_DIALOG (dialog2));
        gtk_widget_destroy (dialog2);
        gtk_main_quit ();
    }     
         
    level_free(level,FALSE);
    gtk_widget_set_sensitive(level->info->back_item,FALSE);  
    
    getLevel(level,nextLevel);  
    setStatus(level);
	if(level->info->persRange) vyberAll(level);
    redrawMap(GTK_WIDGET(level->info->area),level);  
        
   if(result==1) showTip(level);
} 

static void on_load_clicked(GtkButton *button, gpointer user_data)
{
    IdenFile* iden=(IdenFile*)user_data;
	GString* homeFile=getHomeDir();
	StrucFile sf;
	GtkTreeModel *model=gtk_tree_view_get_model (GTK_TREE_VIEW(iden->tree));
    GList *list;
    GtkTreeIter iter;
	gchar* name;
	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (iden->tree));
	gboolean nalezen=FALSE;
	FILE* file;
	gint version;
  
    list=gtk_tree_selection_get_selected_rows(selection,&model);
    if(!list)return ;

    if(!gtk_tree_model_get_iter(model,&iter,(GtkTreePath*)list->data)){
        g_list_free (list);
        return ;
	}
    gtk_tree_model_get (model, &iter,0, &name ,-1);
	g_list_free (list);
    
	if(homeFile){
        g_string_append_c(homeFile,G_DIR_SEPARATOR); 
        g_string_append(homeFile,"games");
        file=fopen(homeFile->str,"rb");
		g_string_free(homeFile,TRUE);
		if(file){
			size_t kontr;

 	    	while(TRUE){
 	    	    kontr=fread((void*)&version,sizeof(gint),1,file);
                if(!kontr)break;
			    kontr=fread((void*)sf.name,sizeof(gchar),32,file);
                if(kontr!=32) break;
				kontr=fread((void*)&sf.type,sizeof(gint),1,file);
                if(!kontr) break;
				kontr=fread((void*)&sf.code,sizeof(guint8),32,file);
                if(kontr!=32)break;
                if(sf.type<0 || sf.type> MAX_FILE ||version != VERSION_FILE[sf.type])
                    continue;
				if(g_ascii_strcasecmp(name,sf.name)==0){
					nalezen=TRUE;
					break; 
				}
			}
			fclose(file);
			if(nalezen){
				GtkWidget* dialog;
	            gint result;
				Score score;
				gchar pom[200];

				g_snprintf(pom,199,_("Are you sure you want to abandon the game now in progress and load this %s type saved game?"),GAME_TYPE[sf.type]);

				if(dekodujScore(&score,sf.code)){
					dialog = gtk_message_dialog_new (GTK_WINDOW(window),
							 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
							 GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,"%s",
							 pom);
					result=gtk_dialog_run (GTK_DIALOG (dialog));
					gtk_widget_destroy (dialog);
					if(result!=GTK_RESPONSE_YES){
					    g_free(name);
                        return;
                    }    

					set_level_set((gpointer)iden->level , sf.type);
					gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(iden->level->info->level_sets_items[sf.type]),TRUE);

        			iden->level->info->file=sf.type;
        			iden->level->best=0;
        			iden->level->info->code=score.code;
        			iden->level->info->por=score.por;
        			iden->level->info->iscore=score.score;
        			iden->level->level=score.level;
        
        			iden->level->lastHistory=0;
                    iden->level->oznacB=NULL;
                    iden->level->oznacP=NULL;
                    iden->level->isPlayerO=FALSE;
                    level_free(iden->level,FALSE);
        			gtk_widget_set_sensitive(iden->level->info->back_item,FALSE);  
        			getLevel(iden->level,iden->level->level);
        			setStatus(iden->level);
                    zapisStav(iden->level);
                    if(iden->level->info->persRange) vyberAll(iden->level);
                    redrawMap(GTK_WIDGET(iden->level->info->area),iden->level);
				}
			}
		}
	}
	g_free(name);

	gtk_dialog_response (GTK_DIALOG(iden->dialog),(gint)0);
}

static void on_save_clicked(GtkButton *button, gpointer user_data)
{
    IdenFile* iden=(IdenFile*)user_data;
	GString* homeFile=getHomeDir();
	StrucFile* sf;
    GList *list,*pom;
    gboolean zmena=FALSE;
	const gchar* name=gtk_entry_get_text(GTK_ENTRY(iden->entry));
	GtkWidget* dialog;
	gint result;
	GTimeVal time;
	FILE* file;
	gint version;
      
	list=NULL;
    if(homeFile){
        g_string_append_c(homeFile,G_DIR_SEPARATOR); 
        g_string_append(homeFile,"games");
        file=fopen(homeFile->str,"rb");
		if(file){
			size_t kontr;
             
 	    	while(TRUE){
				sf=(StrucFile*)g_malloc(sizeof(StrucFile));
                kontr=fread((void*)&version,sizeof(gint),1,file);
                if(!kontr)break;
                kontr=fread((void*)sf->name,sizeof(gchar),32,file);
                if(kontr!=32) break;
				kontr=fread((void*)&sf->type,sizeof(gint),1,file);
                if(!kontr) break;
                kontr=fread((void*)&sf->code,sizeof(guint8),32,file);
                if(kontr!=32)break;
                if(sf->type<0 || sf->type> MAX_FILE ||version != VERSION_FILE[sf->type]){
                    g_free(sf);
                    continue;
                }
				if(g_ascii_strcasecmp(name,sf->name)==0){
					dialog = gtk_message_dialog_new (GTK_WINDOW(window),
							 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
							 GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
							 _("Game with this name already exist.\nDo you really want owerload it?"));
					result=gtk_dialog_run (GTK_DIALOG (dialog));
					gtk_widget_destroy (dialog);
					if(result==GTK_RESPONSE_YES){    
						Score score;

                        g_stpcpy (sf->name,name);
						sf->type=iden->level->info->file;

						g_get_current_time(&time);
						score.time=(GTime)time.tv_sec;
						score.code=iden->level->info->code;
						score.por=iden->level->info->por;
						score.score=iden->level->info->iscore;
						score.level=iden->level->level;
						kodujScore(&sf->code,score);
					}else{
					    fclose(file);
					    g_string_free(homeFile,TRUE);
		        		g_list_free (list);
		        		return;
					}    
					zmena=TRUE;
                }
				list=g_list_append(list,(gpointer)sf);
			}
			fclose(file);
        }else{
            sf=(StrucFile*)g_malloc(sizeof(StrucFile));
        }    			
		if(!zmena){
			Score score;

            g_stpcpy (sf->name,name);
		    sf->type=iden->level->info->file;

            g_get_current_time(&time);
			score.time=(GTime)time.tv_sec;
			score.code=iden->level->info->code;
			score.por=iden->level->info->por;
			score.score=iden->level->info->iscore;
			score.level=iden->level->level;
			kodujScore(&sf->code,score);
			list=g_list_append(list,(gpointer)sf);
        }else{
			g_free(sf);
		}
		
		pom=list;
		file=fopen(homeFile->str,"wb");
		g_string_free(homeFile,TRUE);
        if(file){
            gchar text[100];
			while(pom){
				sf=(StrucFile*)pom->data;
				pom=pom->next;
				if(fwrite((void*)&VERSION_FILE[sf->type],sizeof(gint),1,file)==0)return;
			    if(fwrite((void*)sf->name,sizeof(gchar),32,file)==0)return;
			    if(fwrite((void*)&sf->type,sizeof(gint),1,file)==0)return;
			    if(fwrite((void*)&sf->code,sizeof(guint8),32,file)==0)return;
				g_free(sf);
			}
			fclose(file);
			g_snprintf(text,99,_("Game saved as %s"),name);
			dialog = gtk_message_dialog_new (GTK_WINDOW(window),
							 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
							 GTK_MESSAGE_INFO,GTK_BUTTONS_OK,"%s",text);
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);
		}else{
			g_string_free(homeFile,TRUE);
		}
		g_list_free (list);
	}
	
    gtk_dialog_response (GTK_DIALOG(iden->dialog),(gint)0);
}

static void on_delete_clicked(GtkButton *button, gpointer user_data)
{
    IdenFile* iden=(IdenFile*)user_data;
	GString* homeFile=getHomeDir();
	StrucFile* sf;
	GtkTreeModel *model=gtk_tree_view_get_model (GTK_TREE_VIEW(iden->tree));
    GList *list,*pom;
    GtkTreeIter iter;
	gchar* name;
	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (iden->tree));
    FILE* file;
    gint version;
  
    list=gtk_tree_selection_get_selected_rows(selection,&model);
    if(!list)return ;

    if(!gtk_tree_model_get_iter(model,&iter,(GtkTreePath*)list->data)){
        g_list_free (list);
        return ;
	}
    gtk_tree_model_get (model, &iter,0, &name ,-1);
	g_list_free (list);
    
	list=NULL;
    if(homeFile){
        g_string_append_c(homeFile,G_DIR_SEPARATOR); 
        g_string_append(homeFile,"games");
        file=fopen(homeFile->str,"rb");
		if(file){
			size_t kontr;

 	    	while(TRUE){
				sf=(StrucFile*)g_malloc(sizeof(StrucFile));
                kontr=fread((void*)&version,sizeof(gint),1,file);
                if(!kontr)break;
                kontr=fread((void*)sf->name,sizeof(gchar),32,file);
                if(kontr!=32) break;
				kontr=fread((void*)&sf->type,sizeof(gint),1,file);
                if(!kontr) break;
                kontr=fread((void*)&sf->code,sizeof(guint8),32,file);
                if(kontr!=32)break;
                if(sf->type<0 || sf->type> MAX_FILE || version != VERSION_FILE[sf->type]){
                    g_free(sf);
                    continue;
                }
				if(g_ascii_strcasecmp(name,sf->name)==0)
					g_free(sf);
                else
				    list=g_list_append(list,(gpointer)sf);
			}
            g_free(sf);
			fclose(file);

			pom=list;
			file=fopen(homeFile->str,"wb");
		    g_string_free(homeFile,TRUE);
            if(file){
				while(pom){
					sf=(StrucFile*)pom->data;
					pom=pom->next;
					if(fwrite((void*)&VERSION_FILE[sf->type],sizeof(gint),1,file)==0)return;
				    if(fwrite((void*)sf->name,sizeof(gchar),32,file)==0)return;
				    if(fwrite((void*)&sf->type,sizeof(gint),1,file)==0)return;
				    if(fwrite((void*)&sf->code,sizeof(guint8),32,file)==0)return;
					g_free(sf);
				}

			}
			fclose(file);
			g_list_free (list);
		}else{
			g_string_free(homeFile,TRUE);
		}
	}
	g_free(name);

	gtk_dialog_response (GTK_DIALOG(iden->dialog),(gint)1);
}

static void on_cancel_clicked(GtkButton *button, gpointer user_data)
{
    IdenFile* iden=(IdenFile*)user_data;
    gtk_dialog_response (GTK_DIALOG(iden->dialog),(gint)2);
}

static void tree_sel_changed (GtkTreeSelection *selection, gpointer user_data)
{
	IdenFile* iden=(IdenFile*)user_data;
	GtkTreeModel *model=gtk_tree_view_get_model (GTK_TREE_VIEW(iden->tree));
    GList* list;
    GtkTreeIter iter;
    gchar* name;
  
    list=gtk_tree_selection_get_selected_rows(selection,&model);
    if(!list)return ;

    if(!gtk_tree_model_get_iter(model,&iter,(GtkTreePath*)list->data)){
        g_list_free (list);
        return ;
	}
    gtk_tree_model_get (model, &iter,0, &name ,-1);
	gtk_entry_set_text(GTK_ENTRY(iden->entry),name);
	g_free(name);
    g_list_free (list);
	
}

static void create_save(IdenFile* iden)
{
    GtkWidget *dialog_vbox;
    GtkWidget* scroll;
    GtkWidget *dialog_action_area1;
    GtkWidget *saveG;
    GtkWidget *label1;
    GtkWidget *deleteG;
    GtkWidget *cancel;
    GtkListStore *store;
    GtkTreeIter iter;
    GtkTreeViewColumn *col;
    GtkCellRenderer *renderer;
	GString* homeFile=getHomeDir();
	GtkTreeSelection *select;
	gint version;
	FILE* file;
	gchar popis[30];
  
    g_snprintf(popis,29,_("Level - %d"),iden->level->level);
    iden->dialog = gtk_dialog_new ();
    gtk_window_set_transient_for(GTK_WINDOW(iden->dialog),GTK_WINDOW(window));

    gtk_window_set_title (GTK_WINDOW (iden->dialog), _("Save Game"));
    gtk_window_set_modal (GTK_WINDOW (iden->dialog), TRUE);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (iden->dialog), TRUE);
    gtk_widget_set_size_request (GTK_WIDGET (iden->dialog),400, 500);
    
    dialog_vbox = GTK_DIALOG (iden->dialog)->vbox;
    gtk_widget_show (dialog_vbox);
   
    scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scroll);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), scroll, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (scroll), 2);

    iden->tree = gtk_tree_view_new ();
    gtk_widget_show (iden->tree);
    gtk_container_add (GTK_CONTAINER (scroll), iden->tree);

    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes (_("Name"), renderer,
                                                    "text",0,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    col = gtk_tree_view_column_new_with_attributes (_("Type"), renderer,
                                                    "text",1,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    col = gtk_tree_view_column_new_with_attributes (_("Date"), renderer,
                                                    "text", 2,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes (_("Level"), renderer,
                                                    "text", 3,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes (_("Score"), renderer,
                                                    "text", 4,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    renderer = gtk_cell_renderer_text_new();
 
    store = gtk_list_store_new (5,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,
                                G_TYPE_UINT,G_TYPE_UINT);
  
	if(homeFile){
        g_string_append_c(homeFile,G_DIR_SEPARATOR); 
        g_string_append(homeFile,"games");
        file=fopen(homeFile->str,"rb");
		g_string_free(homeFile,TRUE);
        if(file){
			gchar name[32];
			gint type;
            Score score;
            CodeScore code;
			size_t kontr;

 	    	while(TRUE){
				GString* date=g_string_new("");
				struct tm cas;
                time_t ucas;

                kontr=fread((void*)&version,sizeof(gint),1,file);
                if(!kontr)break;
                kontr=fread((void*)name,sizeof(gchar),32,file);
                if(kontr!=32) break;
				kontr=fread((void*)&type,sizeof(gint),1,file);
                if(!kontr) break;
               	kontr=fread((void*)&code,sizeof(guint8),32,file);
                if(kontr!=32) break;
                if(type<0 || type> MAX_FILE ||version != VERSION_FILE[type]) continue;
                if(!dekodujScore(&score,code))break;
				
                ucas=score.time;
				cas=*localtime(&ucas);
				g_string_printf(date,cas.tm_min<10?"%d:0%d %d.%d.%d":"%d:%d %d.%d.%d",
								cas.tm_hour,cas.tm_min,cas.tm_mday,cas.tm_mon+1,
								cas.tm_year+1900);
				gtk_list_store_append(store, &iter);
				gtk_list_store_set (store,&iter,0,name,1,_(GAME_TYPE[type]),2,date->str,3,
									(guint)score.level,4,(guint)score.score,-1);

				g_string_free(date,TRUE);
			}
			fclose(file);
		}
	}

    gtk_tree_view_set_model(GTK_TREE_VIEW(iden->tree),GTK_TREE_MODEL(store));
    g_object_unref (G_OBJECT (store));

	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (iden->tree));
	g_signal_connect (G_OBJECT (select), "changed",
		    G_CALLBACK (tree_sel_changed),(gpointer)iden);
     
    label1 = gtk_label_new (_("Set game name:"));
    gtk_widget_show (label1);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), label1, FALSE,FALSE, 3);
          
    iden->entry=gtk_entry_new_with_max_length(31);     
    gtk_widget_show (iden->entry);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), iden->entry, FALSE, TRUE, 3);
	gtk_entry_set_text(GTK_ENTRY(iden->entry),popis);
    
    dialog_action_area1 = GTK_DIALOG (iden->dialog)->action_area;
    gtk_widget_show (dialog_action_area1);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
    
    saveG = gtk_button_new_with_mnemonic (_("Save Game"));
    gtk_dialog_add_action_widget (GTK_DIALOG (iden->dialog),saveG, 0);
    gtk_widget_show (saveG);
    GTK_WIDGET_SET_FLAGS (saveG, GTK_CAN_DEFAULT);
    gtk_widget_grab_focus(saveG);
    
    g_signal_connect ((gpointer) saveG, "clicked",
                    G_CALLBACK (on_save_clicked),
                    (gpointer)iden);
    
    deleteG = gtk_button_new_with_mnemonic (_("Delete Game"));
    gtk_widget_show (deleteG);
    gtk_dialog_add_action_widget (GTK_DIALOG (iden->dialog), deleteG, 0);
    GTK_WIDGET_SET_FLAGS (deleteG, GTK_CAN_DEFAULT);
    
        
    g_signal_connect ((gpointer) deleteG, "clicked",
                G_CALLBACK (on_delete_clicked),
                (gpointer)iden);
 
    cancel = gtk_button_new_with_mnemonic (_("Cancel"));
    gtk_widget_show (cancel);
    gtk_dialog_add_action_widget (GTK_DIALOG (iden->dialog), cancel, 0);
    GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
        
    g_signal_connect ((gpointer) cancel, "clicked",
                    G_CALLBACK (on_cancel_clicked),
                    (gpointer)iden);
}

static void create_load(IdenFile* iden)
{
    GtkWidget *dialog_vbox;
    GtkWidget* scroll;
    GtkWidget *dialog_action_area1;
    GtkWidget *loadG;
    GtkWidget *deleteG;
    GtkWidget *cancel;
    GtkListStore *store;
    GtkTreeIter iter;
    GtkTreeViewColumn *col;
    GtkCellRenderer *renderer;
	GString* homeFile=getHomeDir();
	FILE* file;
	gint version;
  
    iden->dialog = gtk_dialog_new ();
    gtk_window_set_transient_for(GTK_WINDOW(iden->dialog),GTK_WINDOW(window));

    gtk_window_set_title (GTK_WINDOW (iden->dialog), _("Load Game"));
    gtk_window_set_modal (GTK_WINDOW (iden->dialog), TRUE);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (iden->dialog), TRUE);
    gtk_widget_set_size_request (GTK_WIDGET (iden->dialog),400, 500);
    
    dialog_vbox = GTK_DIALOG (iden->dialog)->vbox;
    gtk_widget_show (dialog_vbox);
   
    scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scroll);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), scroll, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (scroll), 2);

    iden->tree = gtk_tree_view_new ();
    gtk_widget_show (iden->tree);
    gtk_container_add (GTK_CONTAINER (scroll), iden->tree);

    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes (_("Name"), renderer,
                                                    "text",0,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    col = gtk_tree_view_column_new_with_attributes (_("Type"), renderer,
                                                    "text",1,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    col = gtk_tree_view_column_new_with_attributes (_("Date"), renderer,
                                                    "text", 2,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes (_("Level"), renderer,
                                                    "text", 3,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    renderer = gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes (_("Score"), renderer,
                                                    "text", 4,NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(iden->tree), col);
    renderer = gtk_cell_renderer_text_new();
 
    store = gtk_list_store_new (5,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,
                                G_TYPE_UINT,G_TYPE_UINT);
  
	if(homeFile){
        g_string_append_c(homeFile,G_DIR_SEPARATOR); 
        g_string_append(homeFile,"games");
        file=fopen(homeFile->str,"rb");
		g_string_free(homeFile,TRUE);
        if(file){
			gchar name[32];
			gint type;
            Score score;
            CodeScore code;
			size_t kontr;

 	    	while(TRUE){
				GString* date=g_string_new("");
				struct tm cas;
                time_t ucas;

				kontr=fread((void*)&version,sizeof(gint),1,file);
                if(!kontr)break;
                kontr=fread((void*)name,sizeof(gchar),32,file);
                if(kontr!=32) break;
				kontr=fread((void*)&type,sizeof(gint),1,file);
                if(!kontr) break;
				kontr=fread((void*)&code,sizeof(guint8),32,file);
                if(kontr!=32) break;
                if(type<0 || type> MAX_FILE || version != VERSION_FILE[type]) continue;
                if(!dekodujScore(&score,code))break;
				
                ucas=score.time;
				cas=*localtime(&ucas);
				g_string_printf(date,cas.tm_min<10?"%d:0%d %d.%d.%d":"%d:%d %d.%d.%d",
								cas.tm_hour,cas.tm_min,cas.tm_mday,cas.tm_mon+1,
								cas.tm_year+1900);
				gtk_list_store_append(store, &iter);
				gtk_list_store_set (store,&iter,0,name,1,_(GAME_TYPE[type]),2,date->str,3,
									(guint)score.level,4,(guint)score.score,-1);

				g_string_free(date,TRUE);
			}
			fclose(file);
		}
	}

    gtk_tree_view_set_model(GTK_TREE_VIEW(iden->tree),GTK_TREE_MODEL(store));
    g_object_unref (G_OBJECT (store));

    dialog_action_area1 = GTK_DIALOG (iden->dialog)->action_area;
    gtk_widget_show (dialog_action_area1);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
    
    loadG = gtk_button_new_with_mnemonic (_("Load Game"));
    gtk_dialog_add_action_widget (GTK_DIALOG (iden->dialog),loadG, 0);
    gtk_widget_show (loadG);
    GTK_WIDGET_SET_FLAGS (loadG, GTK_CAN_DEFAULT);
    
    g_signal_connect ((gpointer) loadG, "clicked",
                    G_CALLBACK (on_load_clicked),
                    (gpointer)iden);
    
    deleteG = gtk_button_new_with_mnemonic (_("Delete Game"));
    gtk_widget_show (deleteG);
    gtk_dialog_add_action_widget (GTK_DIALOG (iden->dialog), deleteG, 0);
    GTK_WIDGET_SET_FLAGS (deleteG, GTK_CAN_DEFAULT);
    
        
    g_signal_connect ((gpointer) deleteG, "clicked",
                G_CALLBACK (on_delete_clicked),
                (gpointer)iden);
 
    cancel = gtk_button_new_with_mnemonic (_("Cancel"));
    gtk_widget_show (cancel);
    gtk_dialog_add_action_widget (GTK_DIALOG (iden->dialog), cancel, 0);
    GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
    gtk_widget_grab_focus(cancel);
        
    g_signal_connect ((gpointer) cancel, "clicked",
                    G_CALLBACK (on_cancel_clicked),
                    (gpointer)iden);

}

static void on_upload_clicked(GtkButton *button, gpointer user_data)
{
   Level* level=(Level*)user_data;

    GString* g_file=getScoreDir();
    GString* l_file;
    GString* str=g_string_new("http://www.babala.org/index.php?component&option=com_score&Itemid=53&task=add&");

    l_file=g_string_new(g_file->str);
    g_string_append(l_file,"scoreL1");
    g_string_append(g_file,"score1");
    g_string_append(str,_("lang=iso-8859-1&"));
    g_string_append(str,_("ng="));
    g_string_append(str,level->info->name->str);
    g_string_append(str,_("&sc="));
    g_string_append(str,g_file->str);
    g_string_append(str,_("&sl="));
    g_string_append(str,l_file->str);
    
    otevriStranku(str);

    g_string_free(str,TRUE);
    g_string_free(l_file,TRUE);
    g_string_free(g_file,TRUE);
}

static void on_cancel_clicked2(GtkButton *button, gpointer user_data)
{
    GtkWidget *dialog=(GtkWidget*)user_data;
    gtk_dialog_response (GTK_DIALOG(dialog),(gint)0);
}

static void uploadScore(Level* level)
{
    GtkWidget* dialog;
    GtkWidget* label;
    GtkWidget *dialog_vbox;
    GtkWidget *dialog_action_area1;
    GtkWidget *table;
    GtkWidget *upload;
    GtkWidget *cancel;
    GString* g_file=getScoreDir();
    GString* l_file;

    l_file=g_string_new(g_file->str);
    g_string_append(l_file,"scoreL1");
    g_string_append(g_file,"score1");

    dialog = gtk_dialog_new ();
    gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(window));

    gtk_window_set_title (GTK_WINDOW (dialog), _("Upload Score"));
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
    
    dialog_vbox = GTK_DIALOG (dialog)->vbox;
    gtk_widget_show (dialog_vbox);

    label = gtk_label_new (_("Data for game score uploading:"));
    gtk_box_pack_start (GTK_BOX (dialog_vbox), label, TRUE, TRUE, 2);
    gtk_widget_show (label);

    label = gtk_label_new (_("( http://www.babala.org/index.php/component/option,com_score/Itemid,53/task,add/ )"));
    gtk_box_pack_start (GTK_BOX (dialog_vbox), label, TRUE, TRUE, 2);
    gtk_widget_show (label);
   
    table = gtk_table_new (3,2,FALSE);
    gtk_widget_show (table);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), table, FALSE, FALSE, 10);

    setCell(table,0,1,0,1,_("Player name"),TRUE);
    setCell(table,0,1,1,2,_("Game score file"),TRUE);
    setCell(table,0,1,2,3,_("Level score file"),TRUE);

    setCell(table,1,2,0,1,level->info->name->str,TRUE);
    setCell(table,1,2,1,2,g_file->str,TRUE);
    setCell(table,1,2,2,3,l_file->str,TRUE);

    dialog_action_area1 = GTK_DIALOG (dialog)->action_area;
    gtk_widget_show (dialog_action_area1);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
    
    upload = gtk_button_new_with_mnemonic (_("_Try upload directly"));
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog),upload, 0);
    gtk_widget_show (upload);
    GTK_WIDGET_SET_FLAGS (upload, GTK_CAN_DEFAULT);
    gtk_widget_grab_focus(upload);
    
    g_signal_connect ((gpointer) upload, "clicked",
                    G_CALLBACK (on_upload_clicked),
                    (gpointer)level);
    
    cancel = gtk_button_new_with_mnemonic (_("Cancel"));
    gtk_widget_show (cancel);
    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), cancel, 0);
    GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);
        
    g_signal_connect ((gpointer) cancel, "clicked",
                    G_CALLBACK (on_cancel_clicked2),
                    (gpointer)dialog);

    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);  
    
    g_string_free(l_file,TRUE);
    g_string_free(g_file,TRUE);
}
    
static void destroy( GtkWidget *widget, gpointer  user_data )
{
    gtk_main_quit ();
}

static void unrealize( GtkWidget *widget, gpointer  user_data )
{
    Level* level=(Level*)user_data;
    zapisName(level);
}

static void menuitem_new( gpointer user_data )
{
    Level* level=(Level*)user_data;
    gint result;
    GtkWidget* dialog;
    
    dialog = gtk_message_dialog_new (GTK_WINDOW(window),
		     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			 GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
			 _("Do you really want to restart game?\n This will abort the game sequence in progress."));
	gtk_dialog_add_button (GTK_DIALOG(dialog),(_("Save & Yes")),1);		 
    result=gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
    if(result==1){
        menuitem_save( user_data );
    }    
    if(result==GTK_RESPONSE_YES || result==1){
        GRand* rand;
        level->info->por=level->info->maxPor;
        level->info->maxPor++;
        rand=g_rand_new ();
        level->info->code=g_rand_int(rand);
        g_rand_free (rand);
        level_free(level,FALSE);
        level->best=0;
        getLevel(level,1);
        level->info->iscore=(guint32)0;
        gtk_widget_set_sensitive(level->info->skip_item,level->info->iscore>49);
        setStatus(level);
        zapisStav(level);
		if(level->info->persRange)vyberAll(level);
        redrawMap(GTK_WIDGET(level->info->area),level);
        showTip(level);
    }      
}

static void menuitem_restart( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    gint nextLevel;
    
    nextLevel=level->level;
         
    level_free(level,FALSE);
    getLevel(level,nextLevel);  
    setStatus(level);
	if(level->info->persRange)	vyberAll(level);
    redrawMap(GTK_WIDGET(level->info->area),level);
}

static void menuitem_skip( gpointer user_data )
{
    Level* level=(Level*)user_data;
    gint result;
    GtkWidget* dialog;
    
    if(level->info->iscore<50)return;
    dialog = gtk_message_dialog_new (GTK_WINDOW(window),
		     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			 GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
			 _("Do you really want skip level?\n Your score will be penalise 50 points"));
			 
    result=gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
    if(result==GTK_RESPONSE_YES){
        level->info->iscore-=50;
        gtk_widget_set_sensitive(level->info->skip_item,level->info->iscore>49);
        level->level++;
        level->best=0;
        zapisStav(level);
        if(level->level==COUNT_LEVEL[level->info->file]+1){
            GtkWidget* dialog2;
            GRand* rand;
            
            level_free(level,FALSE);
            level->info->por=level->info->maxPor;
            level->info->maxPor++;
            level->info->iscore=(guint32)0;
            rand=g_rand_new ();
            level->info->code=g_rand_int(rand);
            g_rand_free (rand);
            getLevel(level,1);
            
            setStatus(level);
            zapisStav(level);
            dialog2 =gtk_message_dialog_new (GTK_WINDOW(window),
        		     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
        			 GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,
        			 _("YOU AR WINNER !!!!"));
            gtk_dialog_run (GTK_DIALOG (dialog2));
            gtk_widget_destroy (dialog2);
            gtk_main_quit ();
        }     
        level_free(level,FALSE);
        gtk_widget_set_sensitive(level->info->back_item,FALSE);  
       
        getLevel(level,level->level);  
        setStatus(level);
	    if(level->info->persRange) vyberAll(level);
        redrawMap(GTK_WIDGET(level->info->area),level);  
    }    
}

static void menuitem_load( gpointer user_data )
{
    Level* level=(Level*)user_data;
	gint result=1;
	IdenFile iden;

    iden.level=level; 
	while(result==1){
		create_load(&iden);
		result =gtk_dialog_run (GTK_DIALOG (iden.dialog));
        gtk_widget_destroy (iden.dialog);
	}
	gtk_window_present (GTK_WINDOW(window));
	gtk_widget_set_sensitive(level->info->skip_item,level->info->iscore>49);
}

static void menuitem_save( gpointer user_data )
{
    Level* level=(Level*)user_data;        
	gint result=1;
	IdenFile iden;

	iden.level=level;
	while(result==1){
		create_save(&iden);
		result =gtk_dialog_run (GTK_DIALOG (iden.dialog));
        gtk_widget_destroy (iden.dialog);
	}
	gtk_window_present (GTK_WINDOW(window));
}

static void menuitem_quit( gpointer user_data )
{
    gtk_main_quit ();
}

static void menuitem_help( gpointer user_data )
{
    if(g_ascii_strcasecmp(_("EN"),"CZ")==0){
        editFileInDialog("helpcz",_("Help"));
    }else{
        if(g_ascii_strcasecmp(_("EN"),"DE")==0){
            editFileInDialog("helpde",_("Help"));
        }else{    
            editFileInDialog("help",_("Help"));
        }    
    }    
}

static void menuitem_legend( gpointer user_data )
{
    editLegend();
}

static void menuitem_license( gpointer user_data )
{
    editFileInDialog("license",_("License"));
}

static void menuitem_about( gpointer user_data )
{
    GtkWidget* dialog;
    dialog = gtk_message_dialog_new (GTK_WINDOW(window),
		     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			 GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,
			 _("BABALA\nversion 0.5.0\nmail:babala@babala.org\nhttp://www.babala.org\ncopyright (c) Pavel Richter 2004-2011"));
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
}

static void menuitem_back( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    history_back(level);
    redrawMap(GTK_WIDGET(level->info->area),level);  
    if(is_history(level)) gtk_widget_set_sensitive(level->info->back_item,TRUE);
    else gtk_widget_set_sensitive(level->info->back_item,FALSE);
    setStatus(level);    
}

static void menuitem_range( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    vyberAll(level);
    redrawMap(GTK_WIDGET(level->info->area),level);   
}

static void menuitem_scale( gpointer user_data )
{
    Level* level=(Level*)user_data;
	guint scale=32;

	if(GTK_CHECK_MENU_ITEM(level->info->scale_item)->active){
		scale=20;
		level->info->less_scale=TRUE;
	}else{
		level->info->less_scale=FALSE;
	}
    
    setRozmer(level,scale);
    zapisName(level);
    redrawMap(GTK_WIDGET(level->info->area),level);   
}

static void menuitem_standart1( gpointer user_data )
{
	set_level_set(user_data , 0);
}

static void menuitem_standart2( gpointer user_data )
{
	set_level_set(user_data , 1);
}

/*static void menuitem_hard( gpointer user_data )
{
	set_level_set(user_data , 2);
}*/

static void menuitem_state( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    editState(level);
}

static void menuitem_score( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    editScore(level);
}

static void menuitem_scoreL( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    editScoreL(level);
}

#ifndef OPEN_SOURCE

static void menuitem_upload( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    uploadScore(level);
}

#endif

static void menuitem_name( gpointer user_data )
{
    Level* level=(Level*)user_data;
    
    editNameDialog(level,TRUE);    
}

static gboolean set_font(GtkEntry *entry)
{
    PangoLayout *pl;
    
    pl=gtk_entry_get_layout(entry);
    pango_layout_set_font_description(pl,fontE);
 
    return FALSE;
}

static gboolean smer(Level* level, guint32 orientation)
{
    gint i=jdi(orientation,level);
    if(i>P_OK_NIC){
        if(i==P_OK_TAH){
            history_add(level);
            if(level->oznacB || level->isPlayerO){
                if(level->oznacB) restoreVyberyB(level);
                if(level->isPlayerO) restoreVyberyP(level);
                    redrawMap(GTK_WIDGET(level->info->area),level);
            }
            if(!level->balabans || !g_list_length(level->balabans)){
                ukonciLevel(TRUE,level,TRUE);
                return FALSE;
            }   
            if(!is_tah(level))
                redrawMap(GTK_WIDGET(level->info->area),level);  
        }else{
            ukonciLevel(P_OK_PLAYER_VYTEZ==i,level,TRUE); 
        }       
    }    
    return TRUE;    
}    

static gboolean strela(Level* level)
{
    gint i=strilej(level);
    if(i>P_OK_NIC){
         if(i==P_OK_TAH){
             history_add(level);
             if(level->oznacB || level->isPlayerO){
             if(level->oznacB) restoreVyberyB(level);
             if(level->isPlayerO) restoreVyberyP(level);
                 redrawMap(GTK_WIDGET(level->info->area),level);
             }
             if(!level->balabans || !g_list_length(level->balabans)){
                 ukonciLevel(TRUE,level,TRUE);
                 return FALSE;
             }
             if(!is_tah(level)) 
                 redrawMap(GTK_WIDGET(level->info->area),level);  
         }else{
             ukonciLevel(P_OK_PLAYER_VYTEZ==i,level,TRUE);
         }    
     }
     return TRUE;        
}

static void mina(Level* level)
{
   if(polozMinu(level)==P_OK_TAH){
       history_add(level);
       if(level->oznacB || level->isPlayerO){
           if(level->oznacB) restoreVyberyB(level);
           if(level->isPlayerO) restoreVyberyP(level);
           redrawMap(GTK_WIDGET(level->info->area),level);
       }
       if(!is_tah(level)) 
           redrawMap(GTK_WIDGET(level->info->area),level);  
    }      
}

static void back(Level* level)
{
    history_back(level);
    if(level->oznacB || level->isPlayerO){
        if(level->oznacB) restoreVyberyB(level);
        if(level->isPlayerO) restoreVyberyP(level);
    }
    redrawMap(GTK_WIDGET(level->info->area),level);      
}

static void next(Level* level)
{
    if(level->oznacB || level->oznacP || level->isPlayerO)
        uvolniOznaceni(level);
	redrawMap(GTK_WIDGET(level->info->area),level);
    if(!hraj_balabans(level)){
        ukonciLevel(FALSE,level,TRUE);
        return;
    }    
    if(!konec_tahu(level)){
        ukonciLevel(TRUE,level,TRUE);
        return;
    }
    if(!level->endFunc(level)){
        ukonciLevel(FALSE,level,FALSE);
        return;
    }
    history_free(level); 
    history_add(level);
	if(level->info->persRange)vyberAll(level);
	redrawMap(GTK_WIDGET(level->info->area),level);   
	gtk_widget_set_sensitive(level->info->back_item,FALSE);       
    setStatus(level);
}


static void on_buttons_func( GtkWidget *widget,gpointer user_data )
{
    ButPar* bp=(ButPar*)user_data;
    
    switch(bp->par){
            case 0:
                if(!smer(bp->level, 3)) return;
                break;
            case 1:
                if(!smer(bp->level, 0)) return;
                break;
            case 3:
                if(!smer(bp->level, 1)) return;
                break;
            case 2:
                if(!smer(bp->level, 2)) return;
                break;
            case 4:
                if(!strela(bp->level))return;
                break;
            case 5:
                mina(bp->level);
                break;
            case 7:
                next(bp->level);
                return;
            case 6:
                back(bp->level);
                break; 
        }
        if(is_history(bp->level)) gtk_widget_set_sensitive(bp->level->info->back_item,TRUE);
        else gtk_widget_set_sensitive(bp->level->info->back_item,FALSE);
        setStatus(bp->level);
}
static gboolean on_mapImage_expose_event(GtkWidget *widget,
                                         GdkEventExpose *event,
                                         gpointer user_data)
{
    drawWholeMap(widget,event,user_data);  
    return FALSE;
}

gboolean
on_area_button_press_event     (GtkWidget *widget,
                                GdkEventButton  *event,
                                gpointer user_data)
{
    Level* level=(Level*)user_data;
    gint x,y;
	gint rozmer=level->info->less_scale?20:32;
    
    x=(guint)(event->x-0.5)/rozmer;
    y=(guint)(event->y-0.5)/rozmer;
    
    if(is_balaban(level,x,y)){
        vyberBalaban(level,INDEX(x,y));
        redrawMap(GTK_WIDGET(level->info->area),level);  
    }
    
    if(level->player.column==x && level->player.raw==y){
        vyberPlayer(level,INDEX(x,y));
        redrawMap(GTK_WIDGET(level->info->area),level);      
    }  
    return FALSE;  
}

     
static gboolean on_key_event(GtkWidget *widget,GdkEventKey *event,
                                      gpointer user_data)
{
    Level* level=(Level*)user_data;
#if (GTK_CHECK_VERSION(2,8,0))
	GtkRange * range;
#endif

    if(event->state & GDK_MOD1_MASK){
        switch(event->keyval){
            case GDK_Left:
#if (GTK_CHECK_VERSION(2,8,0))
		 		range = GTK_RANGE(gtk_scrolled_window_get_hscrollbar(level->info->scroll));
		 		g_signal_emit_by_name(G_OBJECT(range), "move-slider", GTK_SCROLL_STEP_LEFT, TRUE);
#else
                g_signal_emit_by_name (G_OBJECT (level->info->scroll), "scroll-child",GTK_SCROLL_STEP_LEFT,TRUE);
#endif
                break;
            case GDK_Up :
#if (GTK_CHECK_VERSION(2,8,0))
		 		range = GTK_RANGE(gtk_scrolled_window_get_vscrollbar(level->info->scroll));
		 		g_signal_emit_by_name(G_OBJECT(range), "move-slider", GTK_SCROLL_STEP_UP, TRUE);
#else
                g_signal_emit_by_name (G_OBJECT (level->info->scroll),"scroll-child",GTK_SCROLL_STEP_UP,TRUE);
#endif
                break;
            case GDK_Right:
#if (GTK_CHECK_VERSION(2,8,0))
		 		range = GTK_RANGE(gtk_scrolled_window_get_hscrollbar(level->info->scroll));
		 		g_signal_emit_by_name(G_OBJECT(range), "move-slider", GTK_SCROLL_STEP_RIGHT, TRUE);
#else
                g_signal_emit_by_name (G_OBJECT (level->info->scroll),"scroll-child",GTK_SCROLL_STEP_RIGHT,TRUE);
#endif
                break;
            case GDK_Down:
#if (GTK_CHECK_VERSION(2,8,0))
		 		range = GTK_RANGE(gtk_scrolled_window_get_vscrollbar(level->info->scroll));
		 		g_signal_emit_by_name(G_OBJECT(range), "move-slider", GTK_SCROLL_STEP_DOWN, TRUE);
#else
                g_signal_emit_by_name (G_OBJECT (level->info->scroll),"scroll-child",GTK_SCROLL_STEP_DOWN,TRUE);
#endif
                break;
            case GDK_R:
            case GDK_r:
                {
                    GtkWidget* dialog;
                    gint result;
                    
                    dialog = gtk_message_dialog_new (GTK_WINDOW(window),
							 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
							 GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
							 _("Do you really want restart actual level?"));
					result=gtk_dialog_run (GTK_DIALOG (dialog));
					gtk_widget_destroy (dialog);
					if(result==GTK_RESPONSE_YES){    
                        menuitem_restart(user_data);
                    }
                }        
                break;
        }    
    }else{
#ifdef WIN 
        int major=gtk_major_version;
        int minor=gtk_minor_version;
        int micro=gtk_micro_version;
        
		if(gtk_major_version==2 && gtk_minor_version==6 && gtk_micro_version>7){
			if(event->keyval==16777215) mina(level);
		}
#endif		
    	switch(event->keyval){
			case GDK_R:
        	case GDK_r:
	            menuitem_range(user_data);
	            break;
	    	case GDK_S:
            case GDK_s:
	            menuitem_state(user_data);
	            break;
            case GDK_Left:
                if(!smer(level, 3)) return FALSE;
                break;
            case GDK_Up :
                if(!smer(level, 0)) return FALSE;
                break;
            case GDK_Right:
                if(!smer(level, 1)) return FALSE;
                break;
            case GDK_Down:
                if(!smer(level, 2)) return FALSE;
                break;
            case GDK_space:
                if(!strela(level))return TRUE;
                break;
            case GDK_Control_L:
            case GDK_Control_R:
                mina(level);
                break;
            case GDK_0:
            case GDK_Insert:
	        case GDK_KP_0:
	        case GDK_KP_Insert:
	            if(!porovnejCas(&znacka))return TRUE;
	            g_get_current_time(&znacka);
                next(level);
                return TRUE;
            case GDK_BackSpace:
                back(level);
                break; 
        }
        if(is_history(level)) gtk_widget_set_sensitive(level->info->back_item,TRUE);
        else gtk_widget_set_sensitive(level->info->back_item,FALSE);
        setStatus(level);    
    } 
    if( event->keyval == GDK_space ) return TRUE;
    return FALSE;
}

static gboolean
on_mapImage_configure_event(GtkWidget *widget,GdkEventConfigure *event,
                            gpointer user_data)
{
    Level* level=(Level*)user_data;
    gint dx,dy;

    gint rozmer=level->info->less_scale?20:32;
    dx=rozmer*level->map.columns+1;
    dy=rozmer*level->map.raws+1;

    if(event->width<dx || event->height<dy){
        gtk_widget_set_size_request(widget,dx,dy);
    }

    return FALSE;
}

int main( int   argc,
          char *argv[] )
{
    GtkWidget *vbox,*hbox;
    GtkWidget *label1,*separator;
    GtkWidget *menu1,*new_item,*load_item,*save_item;
    GtkWidget *restart_item,*quit_item,*menu_bar;
    GtkWidget *file_menu;
    GtkWidget *menu2,*help_item,*legend_item,*license_item,*about_item,*help_menu,*sep_item;
    GtkWidget *menu3,*score_item,*scoreL_item,*name_item,*tools_menu,*range_item,*state_item;
#ifndef OPEN_SOURCE
    GtkWidget *upload_item;
#endif

    GtkWidget *menu4,*type_menu;
	GSList *type_group;
    PangoContext* pc;
    Level level;
    Info info;
    gint i;
    GdkPixbuf *ico;
    GdkPixbuf **icon;
    GtkTooltips *tooltips[8];
    ButPar bp[8];
    gboolean zacatek=FALSE;
    
    g_get_current_time(&znacka);
#ifdef WIN
    PangoFontMap* fm;
    {
        bindtextdomain ("babala", "locale");
        bind_textdomain_codeset ("babala", "UTF-8");
        textdomain ("babala");
    }    
#else

#ifdef ENABLE_NLS
#ifdef RELATIVE_PATH
   {
       GString* dir=g_string_new("");
       g_string_append(dir,"..");
       g_string_append_c(dir,G_DIR_SEPARATOR);
       g_string_append(dir,"share");
       g_string_append_c(dir,G_DIR_SEPARATOR);
       g_string_append(dir,"locale");
       bindtextdomain (GETTEXT_PACKAGE,dir->str);
       bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
       textdomain (GETTEXT_PACKAGE);
       g_string_free(dir,TRUE);
   }
#else
    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain (GETTEXT_PACKAGE);
#endif
#endif

#endif
    gtk_init (&argc, &argv);
    
    ico=initDraw();
    level.info=&info;
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    
    if(!nastavZacatek(&level)){
         nastavZacatekNew(&level);
          zacatek=TRUE;
    }    
        
    if(ico)
         gtk_window_set_icon(GTK_WINDOW(window),ico);

    gtk_widget_set_size_request (GTK_WIDGET (window),760,300);
    if(level.info->max) gtk_window_maximize (GTK_WINDOW(window));
    else gtk_window_resize (GTK_WINDOW(window),level.info->windx,level.info->windy);
    gtk_window_set_title (GTK_WINDOW (window), "Babala");

    gtk_widget_set_events(window,GDK_KEY_PRESS_MASK);
    
    g_signal_connect ((gpointer)window, "key_press_event",
                      G_CALLBACK (on_key_event),
                      (gpointer)&level);
                      
    g_signal_connect (G_OBJECT (window), "destroy",
                      G_CALLBACK (destroy), NULL);
                      
    g_signal_connect (G_OBJECT (window), "unrealize",
                      G_CALLBACK (unrealize), (gpointer)&level);
   
    vbox = gtk_vbox_new (FALSE, 0);

    menu1 = gtk_menu_new (); 
    new_item = gtk_menu_item_new_with_label (_("New game"));
    restart_item = gtk_menu_item_new_with_label (_("Restart level"));
    sep_item = gtk_separator_menu_item_new ();
    quit_item = gtk_menu_item_new_with_label (_("Exit Babala"));
    info.skip_item = gtk_menu_item_new_with_label (_("Skip level"));
    load_item = gtk_menu_item_new_with_label (_("Load game"));
    save_item = gtk_menu_item_new_with_label (_("Save game"));

    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), restart_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), info.skip_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), sep_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), new_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), load_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), save_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu1), quit_item);
    gtk_widget_set_sensitive(info.skip_item,info.iscore>49);


    g_signal_connect_swapped (G_OBJECT (new_item), "activate",
        G_CALLBACK (menuitem_new), (gpointer) &level);
    g_signal_connect_swapped (G_OBJECT (restart_item), "activate",
        G_CALLBACK (menuitem_restart), (gpointer) &level);
    g_signal_connect_swapped (G_OBJECT (quit_item), "activate",
        G_CALLBACK (menuitem_quit),(gpointer)NULL);
    g_signal_connect_swapped (G_OBJECT (info.skip_item), "activate",
        G_CALLBACK (menuitem_skip), (gpointer) &level);
    g_signal_connect_swapped (G_OBJECT (load_item), "activate",
        G_CALLBACK (menuitem_load), (gpointer) &level);
    g_signal_connect_swapped (G_OBJECT (save_item), "activate",
        G_CALLBACK (menuitem_save),(gpointer)&level);
    
    file_menu = gtk_menu_item_new_with_label (_("Game"));
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_menu), menu1);

    menu2 = gtk_menu_new (); 
    help_item = gtk_menu_item_new_with_label (_("Help"));
    legend_item = gtk_menu_item_new_with_label (_("Legend"));
    license_item = gtk_menu_item_new_with_label (_("License"));
    about_item = gtk_menu_item_new_with_label (_("About BABALA"));
 
    gtk_menu_shell_append (GTK_MENU_SHELL (menu2), help_item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu2), legend_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu2), license_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu2), about_item);

    g_signal_connect_swapped (G_OBJECT (help_item), "activate",
        G_CALLBACK (menuitem_help), (gpointer) NULL);

	g_signal_connect_swapped (G_OBJECT (legend_item), "activate",
        G_CALLBACK (menuitem_legend), (gpointer) NULL);

    g_signal_connect_swapped (G_OBJECT (license_item), "activate",
        G_CALLBACK (menuitem_license), (gpointer) NULL);
        
    g_signal_connect_swapped (G_OBJECT (about_item), "activate",
        G_CALLBACK (menuitem_about), (gpointer) NULL);
    
    help_menu = gtk_menu_item_new_with_label (_("Help"));
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (help_menu), menu2);

    menu3 = gtk_menu_new (); 
    info.back_item = gtk_menu_item_new_with_label (_("Back"));
    range_item = gtk_menu_item_new_with_label (_("Show Range"));
    info.scale_item = gtk_check_menu_item_new_with_label (_("Smaller Scale"));
    score_item = gtk_menu_item_new_with_label (_("Score"));
    scoreL_item = gtk_menu_item_new_with_label (_("Levels Score"));
#ifndef OPEN_SOURCE
    upload_item = gtk_menu_item_new_with_label (_("Upload Score"));
#endif
    state_item=gtk_menu_item_new_with_label (_("State of Level"));
    name_item = gtk_menu_item_new_with_label (_("Options"));
 
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), info.back_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), range_item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu3), info.scale_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), state_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), score_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), scoreL_item);
#ifndef OPEN_SOURCE
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), upload_item);
#endif
    gtk_menu_shell_append (GTK_MENU_SHELL (menu3), name_item);

    gtk_widget_set_sensitive(info.back_item,FALSE);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(info.scale_item),info.less_scale);

    g_signal_connect_swapped (G_OBJECT (info.back_item), "activate",
        G_CALLBACK (menuitem_back), (gpointer)&level);

    g_signal_connect_swapped (G_OBJECT (range_item), "activate",
        G_CALLBACK (menuitem_range), (gpointer)&level);

	g_signal_connect_swapped (G_OBJECT (info.scale_item), "activate",
        G_CALLBACK (menuitem_scale), (gpointer)&level);

    g_signal_connect_swapped (G_OBJECT (state_item), "activate",
        G_CALLBACK (menuitem_state), (gpointer)&level);

    g_signal_connect_swapped (G_OBJECT (score_item), "activate",
        G_CALLBACK (menuitem_score), (gpointer)&level);
    
    g_signal_connect_swapped (G_OBJECT (scoreL_item), "activate",
        G_CALLBACK (menuitem_scoreL), (gpointer)&level);
#ifndef OPEN_SOURCE
    g_signal_connect_swapped (G_OBJECT (upload_item), "activate",
        G_CALLBACK (menuitem_upload), (gpointer)&level);
#endif       
    g_signal_connect_swapped (G_OBJECT (name_item), "activate",
        G_CALLBACK (menuitem_name), (gpointer)&level);
    
    tools_menu = gtk_menu_item_new_with_label (_("Tools"));
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (tools_menu), menu3);

    menu4 = gtk_menu_new (); 
	type_group=NULL;  
    info.level_sets_items[0] = gtk_radio_menu_item_new_with_label (type_group,_("Standard"));
	type_group=gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(info.level_sets_items[0]));

	info.level_sets_items[1] = gtk_radio_menu_item_new_with_label (type_group,_("Advanced"));
	type_group=gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(info.level_sets_items[1]));

/*    info.level_sets_items[2] = gtk_radio_menu_item_new_with_label (type_group,_("Hard"));
	type_group=gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(info.level_sets_items[2]));*/

    gtk_menu_shell_append (GTK_MENU_SHELL (menu4), info.level_sets_items[0]);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu4), info.level_sets_items[1]);
//	gtk_menu_shell_append (GTK_MENU_SHELL (menu4), info.level_sets_items[2]);

    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(info.level_sets_items[info.file]),TRUE);

    g_signal_connect_swapped(G_OBJECT (info.level_sets_items[0]), "activate",
        G_CALLBACK (menuitem_standart1), (gpointer)&level);

    g_signal_connect_swapped (G_OBJECT (info.level_sets_items[1]), "activate",
        G_CALLBACK (menuitem_standart2), (gpointer)&level);

/*    g_signal_connect_swapped (G_OBJECT (info.level_sets_items[2]), "activate",
        G_CALLBACK (menuitem_hard), (gpointer)&level);*/

    type_menu = gtk_menu_item_new_with_label (_("Level Set"));
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (type_menu), menu4);
               
    menu_bar = gtk_menu_bar_new ();
    gtk_box_pack_start (GTK_BOX (vbox), menu_bar, FALSE, FALSE, 0);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), file_menu);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), type_menu);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), tools_menu);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), help_menu);

    info.scroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new (NULL, NULL));
    gtk_scrolled_window_set_policy(info.scroll,
        GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(info.scroll), TRUE, TRUE, 0);

    info.viewport =GTK_VIEWPORT(gtk_viewport_new (NULL, NULL));
    gtk_container_add (GTK_CONTAINER (info.scroll), GTK_WIDGET(info.viewport));

    info.area = GTK_DRAWING_AREA(gtk_drawing_area_new ());
    gtk_container_add (GTK_CONTAINER (info.viewport), GTK_WIDGET(info.area));
    gtk_widget_set_events(GTK_WIDGET(info.area),GDK_BUTTON_PRESS_MASK);
    g_signal_connect ((gpointer)info.area, "button_press_event",
                    G_CALLBACK (on_area_button_press_event),
                    (gpointer)&level);
  
    g_signal_connect ((gpointer) info.area, "expose_event",
                      G_CALLBACK (on_mapImage_expose_event),
                      (gpointer)&level);

    g_signal_connect ((gpointer) info.area, "configure_event",
                      G_CALLBACK (on_mapImage_configure_event),
                      (gpointer)&level);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 2);
    
    icon=(GdkPixbuf**)g_malloc(sizeof(GdkPixbuf*)*8);
    loadButtonPixbuf(icon);
    
    for(i=0;i<8;i++){
        GtkWidget *box;
        GtkWidget *image;

        info.butt[i]=gtk_button_new ();
        
        bp[i].level=&level;
        bp[i].par=i;
        g_signal_connect (G_OBJECT (info.butt[i]), "clicked",
            G_CALLBACK (on_buttons_func), (gpointer)&(bp[i]));

        if(icon[i]){
            box = gtk_hbox_new (FALSE, 0);
            gtk_container_set_border_width (GTK_CONTAINER (box), 0);
            image = gtk_image_new_from_pixbuf (icon[i]);
            gtk_box_pack_start (GTK_BOX (box), image, TRUE, TRUE, 0);
            gtk_widget_show (image);
            gtk_widget_show (box);
	        gtk_container_add (GTK_CONTAINER (info.butt[i]), box);
        }
        tooltips[i] = gtk_tooltips_new ();
        gtk_widget_show (info.butt[i]);
        gtk_box_pack_start (GTK_BOX (hbox), info.butt[i], FALSE, FALSE, 2);
        gtk_tooltips_set_tip (tooltips[i], info.butt[i], _(TOOLTIPS[i]), NULL);
    }
    for(i=0;i<8;i++) gdk_pixbuf_unref(icon[i]);
    g_free((gpointer)icon);
    
    separator = gtk_vseparator_new(); 
    gtk_box_pack_start (GTK_BOX (hbox), separator, FALSE, FALSE, 2);   
  
    label1 = gtk_label_new (_("Moves:/Shots:"));
    gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 4);
    
    info.moves1=GTK_ENTRY(gtk_entry_new_with_max_length(1));

#ifdef WIN
    fm=(PangoFontMap*)pango_win32_font_map_for_display ();     
    pc=(PangoContext*)pango_context_new ();
    pango_context_set_font_map(pc,fm);
#else
    pc=gtk_widget_create_pango_context(GTK_WIDGET(level.info->moves1));
#endif
    fontE=pango_font_description_copy(pango_context_get_font_description(pc));
	fontEB=pango_font_description_copy(pango_context_get_font_description(pc));
	fontB=pango_font_description_copy(pango_context_get_font_description(pc));
    pango_font_description_set_family(fontE,"Sans");
    pango_font_description_set_weight(fontE,PANGO_WEIGHT_BOLD);
	pango_font_description_set_family(fontEB,"Sans");
    pango_font_description_set_weight(fontEB,PANGO_WEIGHT_BOLD);
 	pango_font_description_set_family(fontB,"Ariel,Sans");
    pango_font_description_set_weight(fontB,PANGO_WEIGHT_BOLD);
	i=pango_font_description_get_size(fontEB);
	i=i*4/3;
	pango_font_description_set_size(fontEB,i);
	
#ifdef WIN 
    pango_font_description_set_size(fontB,PANGO_SCALE*10);
    pango_win32_shutdown_display () ; 
#endif
     
    g_signal_connect(info.moves1, "expose-event", G_CALLBACK(set_font), info.moves1);
                        
    gtk_editable_set_editable( GTK_EDITABLE(info.moves1), FALSE);
    gtk_widget_set_size_request(GTK_WIDGET(info.moves1),18,22);
    GTK_WIDGET_UNSET_FLAGS(info.moves1,GTK_CAN_FOCUS);
    gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(info.moves1), FALSE, FALSE, 2);

    label1 = gtk_label_new ("/");
    gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 2);
    
    info.moves2=GTK_ENTRY(gtk_entry_new_with_max_length(1));
    
    g_signal_connect(info.moves2, "expose-event", G_CALLBACK(set_font), info.moves2);
       
    gtk_editable_set_editable( GTK_EDITABLE(info.moves2), FALSE);
    gtk_widget_set_size_request(GTK_WIDGET(info.moves2),18,22);
    GTK_WIDGET_UNSET_FLAGS(info.moves2,GTK_CAN_FOCUS);
    gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(info.moves2), FALSE, FALSE, 2);

    label1 = gtk_label_new (_("Projectiles:/Mines:"));
    gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 4);
    
    info.projectile1=GTK_ENTRY(gtk_entry_new_with_max_length(3));
    
    g_signal_connect(info.projectile1, "expose-event", G_CALLBACK(set_font), info.projectile1);
       
    gtk_editable_set_editable( GTK_EDITABLE(info.projectile1), FALSE);
    gtk_widget_set_size_request(GTK_WIDGET(info.projectile1),35,22);
    GTK_WIDGET_UNSET_FLAGS(info.projectile1,GTK_CAN_FOCUS);
    gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(info.projectile1), FALSE, FALSE, 2);
    
    label1 = gtk_label_new ("/");
    gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 2);

    info.projectile2=GTK_ENTRY(gtk_entry_new_with_max_length(3));
    
    g_signal_connect(info.projectile2, "expose-event", G_CALLBACK(set_font), info.projectile2);
    
    gtk_editable_set_editable( GTK_EDITABLE(info.projectile2), FALSE);
    gtk_widget_set_size_request(GTK_WIDGET(info.projectile2),35,22);
    GTK_WIDGET_UNSET_FLAGS(info.projectile2,GTK_CAN_FOCUS);
    gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(info.projectile2), FALSE, FALSE, 2);
    
    separator = gtk_vseparator_new(); 
    gtk_box_pack_start (GTK_BOX (hbox), separator, FALSE, FALSE, 2);   
    
    label1 = gtk_label_new (_("Level:"));
    gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 3);
    
    info.score1=GTK_ENTRY(gtk_entry_new_with_max_length(5));
    
    g_signal_connect(info.score1, "expose-event", G_CALLBACK(set_font), info.score1);
    
    gtk_editable_set_editable( GTK_EDITABLE(info.score1), FALSE);
    gtk_widget_set_size_request(GTK_WIDGET(info.score1),44,22);
    GTK_WIDGET_UNSET_FLAGS(info.score1,GTK_CAN_FOCUS);
    gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(info.score1), FALSE, FALSE, 2);
    
    label1 = gtk_label_new (_("Score:"));
    gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 3);
    
    info.score2=GTK_ENTRY(gtk_entry_new_with_max_length(9));
    
    g_signal_connect(info.score2, "expose-event", G_CALLBACK(set_font), info.score2);
        
    gtk_editable_set_editable( GTK_EDITABLE(info.score2), FALSE);
    GTK_WIDGET_UNSET_FLAGS(info.score2,GTK_CAN_FOCUS);
    gtk_widget_set_size_request(GTK_WIDGET(info.score2),69,22);
    gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(info.score2), FALSE, FALSE, 2);

     setStatus(&level);
	if(level.info->persRange) vyberAll(&level);
    
    gtk_container_add (GTK_CONTAINER (window), vbox);

    initDrawFont(&level);  

    setRozmer(&level,info.less_scale?20:32);  

    gtk_widget_show_all (window);
    if(zacatek) showTip(&level);
    gtk_main ();

    pango_font_description_free (fontE);
	pango_font_description_free (fontEB);
	pango_font_description_free (fontB);
	g_object_unref((gpointer)pc);
    unrefPixmaps();
    level_free(&level,TRUE);
    
    return 0;
}
